package Swagger2::SchemaValidator;
use Mojo::Base 'JSON::Validator::OpenAPI';
warn "# Swagger2::SchemaValidator is deprecated in favor of JSON::Validator::OpenAPI";
1;

=encoding utf8

=head1 NAME

Swagger2::SchemaValidator - DEPRECATED

=head1 DEPRECATION WARNING

See L<Swagger2>.

=head1 DESCRIPTION

Use L<JSON::Validator::OpenAPI> instead.

=head1 SEE ALSO

L<Swagger2> and L<JSON::Validator::OpenAPI>.

=cut
